@@ -1,38 +1,9 @@
 /*
  * server.c -- nsd(8) network input/output
  *
- * Alexis Yushin, <alexis@nlnetlabs.nl>
- *
  * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
- * This software is an open source.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * Redistributions of source code must retain the above copyright notice,
- * this list of conditions and the following disclaimer.
- *
- * Redistributions in binary form must reproduce the above copyright notice,
- * this list of conditions and the following disclaimer in the documentation
- * and/or other materials provided with the distribution.
- *
- * Neither the name of the NLNET LABS nor the names of its contributors may
- * be used to endorse or promote products derived from this software without
- * specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * See LICENSE for the license.
  *
  */
 
@@ -51,7 +22,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <stddef.h>
-#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -61,15 +31,9 @@
 #include <netdb.h>
 
 #include "axfr.h"
-#include "dns.h"
 #include "namedb.h"
-#include "dname.h"
 #include "netio.h"
-#include "nsd.h"
 #include "plugins.h"
-#include "query.h"
-#include "region-allocator.h"
-#include "util.h"
 
 
 /*
@@ -79,7 +43,7 @@ struct udp_handler_data
 {
 	struct nsd        *nsd;
 	struct nsd_socket *socket;
-	region_type       *query_region;
+	query_type        *query;
 };
 
 /*
@@ -89,7 +53,6 @@ struct udp_handler_data
 struct tcp_accept_handler_data {
 	struct nsd         *nsd;
 	struct nsd_socket  *socket;
-	region_type        *query_region;
 	size_t              tcp_accept_handler_count;
 	netio_handler_type *tcp_accept_handlers;
 };
@@ -124,7 +87,7 @@ struct tcp_handler_data
 	/*
 	 * The current query data for this TCP connection.
 	 */
-	struct query     query;
+	query_type      *query;
 
 	/*
 	 * These fields are used to enable the TCP accept handlers
@@ -380,7 +343,6 @@ server_init(struct nsd *nsd)
 
 	/* Open the database... */
 	if ((nsd->db = namedb_open(nsd->dbfile)) == NULL) {
-		log_msg(LOG_ERR, "unable to load %s: %s", nsd->dbfile, strerror(errno));
 		return -1;
 	}
 
@@ -450,6 +412,7 @@ server_main(struct nsd *nsd)
 	int status;
 	pid_t child_pid;
 	pid_t reload_pid = -1;
+	sig_atomic_t mode;
 	
 	assert(nsd->server_kind == NSD_SERVER_MAIN);
 
@@ -458,8 +421,8 @@ server_main(struct nsd *nsd)
 		exit(1);
 	}
 
-	while (nsd->mode != NSD_SHUTDOWN) {
-		switch (nsd->mode) {
+	while ((mode = nsd->mode) != NSD_SHUTDOWN) {
+		switch (mode) {
 		case NSD_RUN:
 			child_pid = waitpid(0, &status, 0);
 		
@@ -586,7 +549,7 @@ process_query(struct nsd *nsd, struct query *query)
 	
 	callback_args.query = query;
 	callback_args.data = NULL;
-	callback_args.result_code = RCODE_OK;
+	callback_args.result_code = NSD_RC_OK;
 
 	callback_result = query_received_callbacks(&callback_args, NULL);
 	if (callback_result != NSD_PLUGIN_CONTINUE) {
@@ -596,7 +559,7 @@ process_query(struct nsd *nsd, struct query *query)
 	rc = query_process(query, nsd);
 	if (rc == QUERY_PROCESSED) {
 		callback_args.data = NULL;
-		callback_args.result_code = RCODE_OK;
+		callback_args.result_code = NSD_RC_OK;
 
 		callback_result = query_processed_callbacks(
 			&callback_args,
@@ -619,12 +582,10 @@ void
 server_child(struct nsd *nsd)
 {
 	size_t i;
-	sigset_t block_sigmask;
-	sigset_t default_sigmask;
 	region_type *server_region = region_create(xalloc, free);
-	region_type *query_region = region_create(xalloc, free);
 	netio_type *netio = netio_create(server_region);
 	netio_handler_type *tcp_accept_handlers;
+	sig_atomic_t mode;
 	
 	assert(nsd->server_kind != NSD_SERVER_MAIN);
 	
@@ -635,22 +596,6 @@ server_child(struct nsd *nsd)
 		close_all_sockets(nsd->udp, nsd->ifs);
 	}
 	
-	/* Allow sigalarm to get us out of the loop */
-	siginterrupt(SIGALRM, 1);
-
-	/*
-	 * Block signals that modify nsd->mode, which must be tested
-	 * for atomically.  These signals are only unblocked while
-	 * waiting in netio_dispatch below.
-	 */
-	sigemptyset(&block_sigmask);
-	sigaddset(&block_sigmask, SIGHUP);
-	sigaddset(&block_sigmask, SIGILL);
-	sigaddset(&block_sigmask, SIGUSR1);
-	sigaddset(&block_sigmask, SIGINT);
-	sigaddset(&block_sigmask, SIGTERM);
-	sigprocmask(SIG_BLOCK, &block_sigmask, &default_sigmask);
-
 	if (nsd->server_kind & NSD_SERVER_UDP) {
 		for (i = 0; i < nsd->ifs; ++i) {
 			struct udp_handler_data *data;
@@ -659,7 +604,8 @@ server_child(struct nsd *nsd)
 			data = (struct udp_handler_data *) region_alloc(
 				server_region,
 				sizeof(struct udp_handler_data));
-			data->query_region = query_region;
+			data->query = query_create(
+				server_region, compressed_dname_offsets);
 			data->nsd = nsd;
 			data->socket = &nsd->udp[i];
 
@@ -689,7 +635,6 @@ server_child(struct nsd *nsd)
 			data = (struct tcp_accept_handler_data *) region_alloc(
 				server_region,
 				sizeof(struct tcp_accept_handler_data));
-			data->query_region = query_region;
 			data->nsd = nsd;
 			data->socket = &nsd->tcp[i];
 			data->tcp_accept_handler_count = nsd->ifs;
@@ -706,28 +651,22 @@ server_child(struct nsd *nsd)
 	}
 	
 	/* The main loop... */	
-	while (nsd->mode != NSD_QUIT) {
+	while ((mode = nsd->mode) != NSD_QUIT) {
 
 		/* Do we need to do the statistics... */
-		if (nsd->mode == NSD_STATS) {
-			nsd->mode = NSD_RUN;
-
+		if (mode == NSD_STATS) {
 #ifdef BIND8_STATS
 			/* Dump the statistics */
 			bind8_stats(nsd);
 #else /* BIND8_STATS */
 			log_msg(LOG_NOTICE, "Statistics support not enabled at compile time.");
 #endif /* BIND8_STATS */
+
+			nsd->mode = NSD_RUN;
 		}
 
-		/*
-		 * Release all memory allocated while processing the
-		 * previous request.
-		 */
-		region_free_all(query_region);
-
 		/* Wait for a query... */
-		if (netio_dispatch(netio, NULL, &default_sigmask) == -1) {
+		if (netio_dispatch(netio, NULL, NULL) == -1) {
 			if (errno != EINTR) {
 				log_msg(LOG_ERR, "netio_dispatch failed: %s", strerror(errno));
 				break;
@@ -739,7 +678,6 @@ server_child(struct nsd *nsd)
 	bind8_stats(nsd);
 #endif /* BIND8_STATS */
 
-	region_destroy(query_region);
 	region_destroy(server_region);
 	
 	server_shutdown(nsd);
@@ -747,15 +685,15 @@ server_child(struct nsd *nsd)
 
 
 static void
-handle_udp(netio_type *netio ATTR_UNUSED,
+handle_udp(netio_type *ATTR_UNUSED(netio),
 	   netio_handler_type *handler,
 	   netio_event_types_type event_types)
 {
 	struct udp_handler_data *data
 		= (struct udp_handler_data *) handler->user_data;
 	int received, sent;
-	struct query q;
-
+	struct query *q = data->query;
+	
 	if (!(event_types & NETIO_EVENT_READ)) {
 		return;
 	}
@@ -768,46 +706,55 @@ handle_udp(netio_type *netio ATTR_UNUSED,
 	}
 
 	/* Initialize the query... */
-	query_init(&q,
-		   data->query_region,
-		   compressed_dname_offsets,
-		   UDP_MAX_MESSAGE_LEN,
-		   0);
-	
-	if ((received = recvfrom(handler->fd, q.iobuf, QIOBUFSZ, 0, (struct sockaddr *)&q.addr, &q.addrlen)) == -1) {
+	query_reset(q, UDP_MAX_MESSAGE_LEN, 0);
+
+	received = recvfrom(handler->fd,
+			    buffer_begin(q->packet),
+			    buffer_remaining(q->packet),
+			    0,
+			    (struct sockaddr *)&q->addr,
+			    &q->addrlen);
+	if (received == -1) {
 		if (errno != EAGAIN && errno != EINTR) {
 			log_msg(LOG_ERR, "recvfrom failed: %s", strerror(errno));
 			STATUP(data->nsd, rxerr);
 		}
-		return;
-	}
-	q.iobufptr = q.iobuf + received;
-	q.tcp = 0;
-
-	/* Process and answer the query... */
-	if (process_query(data->nsd, &q) != QUERY_DISCARDED) {
-		if (RCODE((&q)) == RCODE_OK && !AA((&q)))
-			STATUP(data->nsd, nona);
-		/* Add edns(0) info if necessary.. */
-		query_addedns(&q, data->nsd);
-
-		if ((sent = sendto(handler->fd, q.iobuf, query_used_size(&q), 0, (struct sockaddr *)&q.addr, q.addrlen)) == -1) {
-			log_msg(LOG_ERR, "sendto failed: %s", strerror(errno));
-			STATUP(data->nsd, txerr);
-			return;
-		} else if (sent != q.iobufptr - q.iobuf) {
-			log_msg(LOG_ERR, "sent %d in place of %d bytes", sent, (int) query_used_size(&q));
-			return;
-		}
-
-#ifdef BIND8_STATS
-		/* Account the rcode & TC... */
-		STATUP2(data->nsd, rcode, RCODE((&q)));
-		if (TC((&q)))
-			STATUP(data->nsd, truncated);
-#endif /* BIND8_STATS */
 	} else {
-		STATUP(data->nsd, dropped);
+		buffer_skip(q->packet, received);
+		buffer_flip(q->packet);
+
+		/* Process and answer the query... */
+		if (process_query(data->nsd, q) != QUERY_DISCARDED) {
+			if (RCODE(q->packet) == RCODE_OK && !AA(q->packet))
+				STATUP(data->nsd, nona);
+
+			/* Add EDNS0 and TSIG info if necessary.  */
+			query_add_optional(q, data->nsd);
+
+			buffer_flip(q->packet);
+
+			sent = sendto(handler->fd,
+				      buffer_begin(q->packet),
+				      buffer_remaining(q->packet),
+				      0,
+				      (struct sockaddr *) &q->addr,
+				      q->addrlen);
+			if (sent == -1) {
+				log_msg(LOG_ERR, "sendto failed: %s", strerror(errno));
+				STATUP(data->nsd, txerr);
+			} else if ((size_t) sent != buffer_remaining(q->packet)) {
+				log_msg(LOG_ERR, "sent %d in place of %d bytes", sent, (int) buffer_remaining(q->packet));
+			} else {
+#ifdef BIND8_STATS
+				/* Account the rcode & TC... */
+				STATUP2(data->nsd, rcode, RCODE(q->packet));
+				if (TC(q->packet))
+					STATUP(data->nsd, truncated);
+#endif /* BIND8_STATS */
+			}
+		} else {
+			STATUP(data->nsd, dropped);
+		}
 	}
 }
 
@@ -832,7 +779,7 @@ cleanup_tcp_handler(netio_type *netio, netio_handler_type *handler)
 	}
 	--data->nsd->current_tcp_count;
 	assert(data->nsd->current_tcp_count >= 0);
-	
+
 	region_destroy(data->region);
 }
 
@@ -844,7 +791,6 @@ handle_tcp_reading(netio_type *netio,
 	struct tcp_handler_data *data
 		= (struct tcp_handler_data *) handler->user_data;
 	ssize_t received;
-	struct query *q = &data->query;
 
 	if (event_types & NETIO_EVENT_TIMEOUT) {
 		/* Connection timed out.  */
@@ -854,16 +800,24 @@ handle_tcp_reading(netio_type *netio,
 
 	assert(event_types & NETIO_EVENT_READ);
 
+	if (data->bytes_transmitted == 0) {
+		query_reset(data->query, TCP_MAX_MESSAGE_LEN, 1);
+	}
+
 	/*
 	 * Check if we received the leading packet length bytes yet.
 	 */
-	if (data->bytes_transmitted < sizeof(q->tcplen)) {
+	if (data->bytes_transmitted < sizeof(uint16_t)) {
 		received = read(handler->fd,
-				(char *) &q->tcplen + data->bytes_transmitted,
-				sizeof(q->tcplen) - data->bytes_transmitted);
+				(char *) &data->query->tcplen
+				+ data->bytes_transmitted,
+				sizeof(uint16_t) - data->bytes_transmitted);
 		if (received == -1) {
 			if (errno == EAGAIN || errno == EINTR) {
-				/* Read would block, wait until more data is available.  */
+				/*
+				 * Read would block, wait until more
+				 * data is available.
+				 */
 				return;
 			} else {
 				log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
@@ -877,7 +831,7 @@ handle_tcp_reading(netio_type *netio,
 		}
 
 		data->bytes_transmitted += received;
-		if (data->bytes_transmitted < sizeof(q->tcplen)) {
+		if (data->bytes_transmitted < sizeof(uint16_t)) {
 			/*
 			 * Not done with the tcplen yet, wait for more
 			 * data to become available.
@@ -885,10 +839,10 @@ handle_tcp_reading(netio_type *netio,
 			return;
 		}
 
-		assert(data->bytes_transmitted == sizeof(q->tcplen));
-
-		q->tcplen = ntohs(q->tcplen);
+		assert(data->bytes_transmitted == sizeof(uint16_t));
 
+		data->query->tcplen = ntohs(data->query->tcplen);
+		
 		/*
 		 * Minimum query size is:
 		 *
@@ -897,26 +851,33 @@ handle_tcp_reading(netio_type *netio,
 		 *   + Query class        (2)
 		 *   + Query type         (2)
 		 */
-		if (q->tcplen < QHEADERSZ + 1 + sizeof(uint16_t) + sizeof(uint16_t)) {
+		if (data->query->tcplen < QHEADERSZ + 1 + sizeof(uint16_t) + sizeof(uint16_t)) {
 			log_msg(LOG_WARNING, "dropping bogus tcp connection");
 			cleanup_tcp_handler(netio, handler);
 			return;
 		}
 
-		if (q->tcplen > q->maxlen) {
+		if (data->query->tcplen > data->query->maxlen) {
 			log_msg(LOG_ERR, "insufficient tcp buffer, dropping connection");
 			cleanup_tcp_handler(netio, handler);
 			return;
 		}
+
+		buffer_set_limit(data->query->packet, data->query->tcplen);
 	}
 
-	assert(data->bytes_transmitted < q->tcplen + sizeof(q->tcplen));
+	assert(buffer_remaining(data->query->packet) > 0);
 
 	/* Read the (remaining) query data.  */
-	received = read(handler->fd, q->iobufptr, q->tcplen - query_used_size(q));
+	received = read(handler->fd,
+			buffer_current(data->query->packet),
+			buffer_remaining(data->query->packet));
 	if (received == -1) {
 		if (errno == EAGAIN || errno == EINTR) {
-			/* Read would block, wait until more data is available.  */
+			/*
+			 * Read would block, wait until more data is
+			 * available.
+			 */
 			return;
 		} else {
 			log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
@@ -930,8 +891,8 @@ handle_tcp_reading(netio_type *netio,
 	}
 
 	data->bytes_transmitted += received;
-	q->iobufptr += received;
-	if (query_used_size(q) < q->tcplen) {
+	buffer_skip(data->query->packet, received);
+	if (buffer_remaining(data->query->packet) > 0) {
 		/*
 		 * Message not yet complete, wait for more data to
 		 * become available.
@@ -939,21 +900,23 @@ handle_tcp_reading(netio_type *netio,
 		return;
 	}
 
-	assert(query_used_size(q) == q->tcplen);
+	assert(buffer_position(data->query->packet) == data->query->tcplen);
 
 	/* Account... */
 #ifndef INET6
 	STATUP(data->nsd, ctcp);
 #else
-	if (data->query.addr.ss_family == AF_INET) {
+	if (data->query->addr.ss_family == AF_INET) {
 		STATUP(data->nsd, ctcp);
-	} else if (data->query.addr.ss_family == AF_INET6) {
+	} else if (data->query->addr.ss_family == AF_INET6) {
 		STATUP(data->nsd, ctcp6);
 	}
 #endif
 
 	/* We have a complete query, process it.  */
-	data->query_state = process_query(data->nsd, q);
+
+	buffer_flip(data->query->packet);
+	data->query_state = process_query(data->nsd, data->query);
 	if (data->query_state == QUERY_DISCARDED) {
 		/* Drop the packet and the entire connection... */
 		STATUP(data->nsd, dropped);
@@ -961,14 +924,17 @@ handle_tcp_reading(netio_type *netio,
 		return;
 	}
 
-	if (RCODE(q) == RCODE_OK && !AA(q)) {
+	if (RCODE(data->query->packet) == RCODE_OK
+	    && !AA(data->query->packet))
+	{
 		STATUP(data->nsd, nona);
 	}
 		
-	query_addedns(q, data->nsd);
+	query_add_optional(data->query, data->nsd);
 
 	/* Switch to the tcp write handler.  */
-	q->tcplen = query_used_size(q);
+	buffer_flip(data->query->packet);
+	data->query->tcplen = buffer_remaining(data->query->packet);
 	data->bytes_transmitted = 0;
 	
 	handler->timeout->tv_sec = TCP_TIMEOUT;
@@ -987,7 +953,7 @@ handle_tcp_writing(netio_type *netio,
 	struct tcp_handler_data *data
 		= (struct tcp_handler_data *) handler->user_data;
 	ssize_t sent;
-	struct query *q = &data->query;
+	struct query *q = data->query;
 
 	if (event_types & NETIO_EVENT_TIMEOUT) {
 		/* Connection timed out.  */
@@ -1032,8 +998,8 @@ handle_tcp_writing(netio_type *netio,
 	assert(data->bytes_transmitted < q->tcplen + sizeof(q->tcplen));
 
 	sent = write(handler->fd,
-		     q->iobuf + data->bytes_transmitted - sizeof(q->tcplen),
-		     query_used_size(q) - data->bytes_transmitted + sizeof(q->tcplen));
+		     buffer_current(q->packet),
+		     buffer_remaining(q->packet));
 	if (sent == -1) {
 		if (errno == EAGAIN || errno == EINTR) {
 			/*
@@ -1048,6 +1014,7 @@ handle_tcp_writing(netio_type *netio,
 		}
 	}
 
+	buffer_skip(q->packet, sent);
 	data->bytes_transmitted += sent;
 	if (data->bytes_transmitted < q->tcplen + sizeof(q->tcplen)) {
 		/*
@@ -1058,13 +1025,17 @@ handle_tcp_writing(netio_type *netio,
 	}
 
 	assert(data->bytes_transmitted == q->tcplen + sizeof(q->tcplen));
-	       
+
 	if (data->query_state == QUERY_IN_AXFR) {
 		/* Continue processing AXFR and writing back results.  */
+		buffer_clear(q->packet);
 		data->query_state = query_axfr(data->nsd, q);
 		if (data->query_state != QUERY_PROCESSED) {
+			query_add_optional(data->query, data->nsd);
+			
 			/* Reset data. */
-			q->tcplen = query_used_size(q);
+			buffer_flip(q->packet);
+			q->tcplen = buffer_remaining(q->packet);
 			data->bytes_transmitted = 0;
 			/* Reset timeout.  */
 			handler->timeout->tv_sec = TCP_TIMEOUT;
@@ -1083,11 +1054,6 @@ handle_tcp_writing(netio_type *netio,
 	 * Done sending, wait for the next request to arrive on the
 	 * TCP socket by installing the TCP read handler.
 	 */
-	query_init(&data->query,
-		   data->query.region,
-		   data->query.compressed_dname_offsets,
-		   data->query.maxlen,
-		   1);
 	data->bytes_transmitted = 0;
 	
 	handler->timeout->tv_sec = TCP_TIMEOUT;
@@ -1132,7 +1098,8 @@ handle_tcp_accept(netio_type *netio,
 	
 	/* Accept it... */
 	addrlen = sizeof(addr);
-	if ((s = accept(handler->fd, (struct sockaddr *)&addr, &addrlen)) == -1) {
+	s = accept(handler->fd, (struct sockaddr *) &addr, &addrlen);
+	if (s == -1) {
 		if (errno != EINTR) {
 			log_msg(LOG_ERR, "accept failed: %s", strerror(errno));
 		}
@@ -1141,6 +1108,7 @@ handle_tcp_accept(netio_type *netio,
 
 	if (fcntl(s, F_SETFL, O_NONBLOCK) == -1) {
 		log_msg(LOG_ERR, "fcntl failed: %s", strerror(errno));
+		close(s);
 		return;
 	}
 	
@@ -1152,23 +1120,16 @@ handle_tcp_accept(netio_type *netio,
 	tcp_data = (struct tcp_handler_data *) region_alloc(
 		tcp_region, sizeof(struct tcp_handler_data));
 	tcp_data->region = tcp_region;
+	tcp_data->query = query_create(tcp_region, compressed_dname_offsets);
 	tcp_data->nsd = data->nsd;
 	
-	query_init(&tcp_data->query,
-		   data->query_region,
-		   compressed_dname_offsets,
-		   (MAX_PACKET_SIZE < data->nsd->tcp_max_msglen
-		    ? MAX_PACKET_SIZE
-		    : data->nsd->tcp_max_msglen),
-		   1);
-
 	tcp_data->tcp_accept_handler_count = data->tcp_accept_handler_count;
 	tcp_data->tcp_accept_handlers = data->tcp_accept_handlers;
 
 	tcp_data->query_state = QUERY_PROCESSED;
 	tcp_data->bytes_transmitted = 0;
-	memcpy(&tcp_data->query.addr, &addr, addrlen);
-	tcp_data->query.addrlen = addrlen;
+	memcpy(&tcp_data->query->addr, &addr, addrlen);
+	tcp_data->query->addrlen = addrlen;
 	
 	tcp_handler = (netio_handler_type *) region_alloc(
 		tcp_region, sizeof(netio_handler_type));
