@@ -1,38 +1,9 @@
 /*
  * util.h -- set of various support routines.
  *
- * Erik Rozendaal, <erik@nlnetlabs.nl>
+ * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
- * Copyright (c) 2003-2004, NLnet Labs. All rights reserved.
- *
- * This software is an open source.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * Redistributions of source code must retain the above copyright notice,
- * this list of conditions and the following disclaimer.
- *
- * Redistributions in binary form must reproduce the above copyright notice,
- * this list of conditions and the following disclaimer in the documentation
- * and/or other materials provided with the distribution.
- *
- * Neither the name of the NLNET LABS nor the names of its contributors may
- * be used to endorse or promote products derived from this software without
- * specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * See LICENSE for the license.
  *
  */
 
@@ -43,6 +14,7 @@
 
 #include <sys/time.h>
 #include <stdarg.h>
+#include <stdio.h>
 #include <syslog.h>
 #include <time.h>
 
@@ -101,6 +73,38 @@ void log_msg(int priority, const char *format, ...)
  */
 void log_vmsg(int priority, const char *format, va_list args);
 
+/*
+ * Set the INDEXth bit of BITS to 1.
+ */
+void set_bit(uint8_t bits[], size_t index);
+
+/*
+ * Set the INDEXth bit of BITS to 0.
+ */
+void clear_bit(uint8_t bits[], size_t index);
+
+/*
+ * Return the value of the INDEXth bit of BITS.
+ */
+int get_bit(uint8_t bits[], size_t index);
+
+/* A general purpose lookup table */
+typedef struct lookup_table lookup_table_type;
+struct lookup_table {
+	int id;
+	const char *name;
+};
+
+/*
+ * Looks up the table entry by name, returns NULL if not found.
+ */
+lookup_table_type *lookup_by_name(lookup_table_type table[], const char *name);
+
+/*
+ * Looks up the table entry by id, returns NULL if not found.
+ */
+lookup_table_type *lookup_by_id(lookup_table_type table[], int id);
+
 /*
  * (Re-)allocate SIZE bytes of memory.  Report an error if the memory
  * could not be allocated and exit the program.  These functions never
@@ -115,7 +119,7 @@ void *xrealloc(void *ptr, size_t size);
  *
  * Returns 0 on failure, 1 on success.
  */
-int write_data(FILE * file, const void *data, size_t size);
+int write_data(FILE *file, const void *data, size_t size);
 
 
 /*
@@ -123,7 +127,7 @@ int write_data(FILE * file, const void *data, size_t size);
  * (big endian).
  */
 static inline void
-copy_uint16(void *dst, uint16_t data)
+write_uint16(void *dst, uint16_t data)
 {
 #ifdef ALLOW_UNALIGNED_ACCESSES
 	* (uint16_t *) dst = htons(data);
@@ -135,7 +139,7 @@ copy_uint16(void *dst, uint16_t data)
 }
 
 static inline void
-copy_uint32(void *dst, uint32_t data)
+write_uint32(void *dst, uint32_t data)
 {
 #ifdef ALLOW_UNALIGNED_ACCESSES
 	* (uint32_t *) dst = htonl(data);
@@ -148,6 +152,32 @@ copy_uint32(void *dst, uint32_t data)
 #endif
 }
 
+/*
+ * Copy data allowing for unaligned accesses in network byte order
+ * (big endian).
+ */
+static inline uint16_t
+read_uint16(const void *src)
+{
+#ifdef ALLOW_UNALIGNED_ACCESSES
+	return ntohs(* (uint16_t *) src);
+#else
+	uint8_t *p = (uint8_t *) src;
+	return (p[0] << 8) | p[1];
+#endif
+}
+
+static inline uint32_t
+read_uint32(const void *src)
+{
+#ifdef ALLOW_UNALIGNED_ACCESSES
+	return ntohl(* (uint32_t *) src);
+#else
+	uint8_t *p = (uint8_t *) src;
+	return (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
+#endif
+}
+
 /*
  * Print debugging information using fprintf(3).
  */
@@ -187,4 +217,39 @@ timeval_to_timespec(struct timespec *left,
 	left->tv_nsec = 1000 * right->tv_usec;
 }
 
+
+/*
+ * Converts a string representation of a period of time into
+ * a long integer of seconds.
+ *
+ * Set the endptr to the first illegal character.
+ *
+ * Interface is similar as strtol(3)
+ *
+ * Returns:
+ *	LONG_MIN if underflow occurs
+ *	LONG_MAX if overflow occurs.
+ *	otherwise number of seconds
+ *
+ * XXX This functions does not check the range.
+ *
+ */
+long strtottl(const char *nptr, const char **endptr);
+
+/*
+ * Convert binary data to a string of hexadecimal characters.
+ */
+ssize_t hex_ntop(uint8_t const *src, size_t srclength, char *target,
+		 size_t targsize);
+
+/*
+ * Strip trailing and leading whitespace from str.
+ */
+void strip_string(char *str);
+
+/*
+ * Convert a single (hexidecimal) digit to its integer value.
+ */
+int hexdigit_to_int(char ch);
+
 #endif /* _UTIL_H_ */
